/* classes: h_files */

#ifndef SCM_COOP_DEFS_H
#define SCM_COOP_DEFS_H

/* Copyright (C) 2002, 2007 Thien-Thi Nguyen
 * Copyright (C) 1996, 1997, 1998, 1999,
 *   2000, 2002 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

/* This file is only included in _scm.h, _qtdl.h, and threads.c.  */

#if 1 /* petrified TIME_WITH_SYS_TIME */
#  include <sys/time.h>
#  include <time.h>
# else
#if 1 /* petrified HAVE_SYS_TIME_H */
#   include <sys/time.h>
#  else
#if 1 /* petrified HAVE_TIME_H */
#    include <time.h>
#   endif
#  endif
# endif

#if 0 /* petrified GUILE_ISELECT */
#include "libguile/iselect.h"
#endif

#if 0 /* petrified GUILE_PTHREAD_COMPAT */
#include <pthread.h>
#endif

/* The coop_t struct must be declared here, since macros in this file
   refer to the data member. */

/* The notion of a thread is merged with the notion of a queue.
   Thread stuff: thread status (sp) and stuff to use during
   (re)initialization.  Queue stuff: next thread in the queue
   (next). */

typedef void *aspp_t;

typedef struct coop_t {
  aspp_t sp;             /* Abstract SP handle. */
  void *sto;             /* `malloc'-allocated stack. */

  struct coop_t *next;    /* Next thread in the queue. */

  struct coop_t *all_next;
  struct coop_t *all_prev;

  void *data;            /* Thread local data */
  void **specific;	 /* Data associated with keys */
  int n_keys;		 /* Upper limit for keys on this thread */

  void *base;            /* Base of stack */
  void *top;             /* Top of stack */

  void *joining;         /* A queue of threads waiting to join this
			    thread */

#if 0 /* petrified GUILE_ISELECT */
  int nfds;
  SCM_SELECT_TYPE *readfds;
  SCM_SELECT_TYPE *writefds;
  SCM_SELECT_TYPE *exceptfds;
  int timeoutp;
  struct timeval wakeup_time;	/* Time to stop sleeping */
  int _errno;
  int retval;
#else
  time_t wakeup_time;    /* Time to stop sleeping */
#endif

#if 0 /* petrified GUILE_PTHREAD_COMPAT */
  pthread_t dummy_thread;
  pthread_mutex_t dummy_mutex;
#endif
} coop_t;

/* A queue is a circular list of threads.  The queue head is a
   designated list element.  If this is a uniprocessor-only
   implementation we can store the `main' thread in this, but in a
   multiprocessor there are several `heavy' threads but only one run
   queue.  A fancier implementation might have private run queues,
   which would lead to a simpler (trivial) implementation */

typedef struct coop_q_t {
  coop_t t;
  coop_t *tail;
} coop_q_t;

/* A Mutex variable is made up of a owner thread, and a queue of threads
   waiting on the mutex */

typedef struct coop_m {
  coop_t *owner;          /* Mutex owner */
  coop_q_t waiting;      /* Queue of waiting threads */
} coop_m;

typedef int coop_mattr;

typedef coop_m scm_mutex_t;

extern int coop_mutex_init (coop_m*);
extern int coop_new_mutex_init (coop_m*, coop_mattr*);
extern int coop_mutex_lock (coop_m*);
extern int coop_mutex_trylock (coop_m*);
extern int coop_mutex_unlock (coop_m*);
extern int coop_mutex_destroy (coop_m*);
#define scm_mutex_init coop_mutex_init
#define scm_mutex_lock coop_mutex_lock
#define scm_mutex_trylock coop_mutex_lock
#define scm_mutex_unlock coop_mutex_unlock
#define scm_mutex_destroy coop_mutex_destroy

/* A Condition variable is made up of a list of threads waiting on the
   condition. */

typedef struct coop_c {
  coop_q_t waiting;      /* Queue of waiting threads */
} coop_c;

typedef int coop_cattr;

typedef coop_c scm_cond_t;

#ifndef HAVE_STRUCT_TIMESPEC
/* POSIX.4 structure for a time value.  This is like a `struct timeval' but
   has nanoseconds instead of microseconds.  */
struct timespec
{
  long int tv_sec;		/* Seconds.  */
  long int tv_nsec;		/* Nanoseconds.  */
};
#endif

extern int coop_condition_variable_init (coop_c*);
extern int coop_new_condition_variable_init (coop_c*, coop_cattr*);
extern int coop_condition_variable_wait_mutex (coop_c*, coop_m*);
extern int coop_condition_variable_timed_wait_mutex (coop_c*,
						     coop_m*,
						     const struct timespec *abstime);
extern int coop_condition_variable_signal (coop_c*);
extern int coop_condition_variable_destroy (coop_c*);
#define scm_cond_init coop_new_condition_variable_init
#define scm_cond_wait coop_condition_variable_wait_mutex
#define scm_cond_timedwait coop_condition_variable_timed_wait_mutex
#define scm_cond_signal coop_condition_variable_signal
#define scm_cond_broadcast coop_condition_variable_signal /* yes */
#define scm_cond_destroy coop_condition_variable_destroy

typedef int coop_k;

typedef coop_k scm_key_t;

extern int coop_key_create (coop_k *keyp, void (*destructor) (void *value));
extern int coop_setspecific (coop_k key, const void *value);
extern void *coop_getspecific (coop_k key);
extern int coop_key_delete (coop_k);
#define scm_key_create coop_key_create
#define scm_setspecific coop_setspecific
#define scm_getspecific coop_getspecific
#define scm_key_delete coop_key_delete

extern coop_t *coop_global_curr;       	/* Currently-executing thread. */

extern void coop_join (coop_t *t);
extern void coop_yield (void);

extern size_t scm_switch_counter;
extern size_t scm_thread_count;



/* Cooperative threads don't need to have these defined */

#define SCM_THREAD_CRITICAL_SECTION_START
#define SCM_THREAD_CRITICAL_SECTION_END



#define SCM_NO_CRITICAL_SECTION_OWNER 0
#define SCM_THREAD_SWITCH_COUNT       50 /* was 10 /mdj */



#define SCM_THREAD_DEFER
#define SCM_THREAD_ALLOW
#define SCM_THREAD_REDEFER
#define SCM_THREAD_REALLOW_1
#define SCM_THREAD_REALLOW_2

#if 0
#define SCM_THREAD_SWITCHING_CODE \
do { \
  if (scm_thread_count > 1) \
    coop_yield (); \
} while (0)

#else
#define SCM_THREAD_SWITCHING_CODE \
do { \
  if (scm_thread_count > 1) \
  { \
    scm_switch_counter--; \
    if (scm_switch_counter == 0) \
      { \
        scm_switch_counter = SCM_THREAD_SWITCH_COUNT; \
        coop_yield (); \
      } \
  } \
} while (0)

#endif

/* For pthreads, this is a value associated with a specific key.
 * For coop, we use a special field for increased efficiency.
 */
#define SCM_THREAD_LOCAL_DATA (coop_global_curr->data)
#define SCM_SET_THREAD_LOCAL_DATA(ptr) (coop_global_curr->data = (ptr))

#endif /* SCM_COOP_DEFS_H */

/* coop-defs.h ends here */
