/* error.h */

#ifndef SCM_ERROR_H
#define SCM_ERROR_H

/* Copyright (C) 2002, 2007 Thien-Thi Nguyen
 * Copyright (C) 1995, 1996, 1997, 1998,
 *   2000, 2002 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */


extern int scm_ints_disabled;



/* GCC can be told that a function doesn't return; this helps it do
   better error checking (for uninitialized variable use, for
   example), and some optimization.  */
#ifdef __GNUC__
#define SCM_NORETURN __attribute__ ((noreturn))
#else
#define SCM_NORETURN
#endif

extern void scm_error (SCM key, const char *subr, const char *message,
                             SCM args, SCM rest) SCM_NORETURN;
extern SCM scm_error_scm (SCM key, SCM subr, SCM message,
                          SCM args, SCM rest) SCM_NORETURN;
extern SCM scm_strerror (SCM err);
extern void scm_syserror (const char *subr) SCM_NORETURN;
extern void scm_syserror_msg (const char *subr, const char *message,
                              SCM args, int eno) SCM_NORETURN;
extern void scm_num_overflow (const char *subr) SCM_NORETURN;
extern void scm_out_of_range (const char *subr, SCM bad_value)
     SCM_NORETURN;
extern void scm_out_of_range_pos (const char *subr, SCM bad_value, SCM pos)
     SCM_NORETURN;
extern void scm_wrong_num_args (SCM proc) SCM_NORETURN;
extern void scm_wrong_type_arg (const char *subr, int pos,
                                SCM bad_value) SCM_NORETURN;
extern void scm_wrong_type_arg_msg (const char *subr, int pos,
                                    SCM bad_value, const char *sz) SCM_NORETURN;
extern void scm_memory_error (const char *subr) SCM_NORETURN;
extern void scm_misc_error (const char *subr, const char *message,
                            SCM args) SCM_NORETURN;
extern SCM scm_wta (SCM arg, const char *pos, const char *s_subr);




#if (SCM_DEBUG_DEPRECATED == 0)

extern void scm_sysmissing (const char *subr) SCM_NORETURN;

#endif  /* SCM_DEBUG_DEPRECATED == 0 */

#endif  /* SCM_ERROR_H */

/* error.h ends here */
