/* gsubr.h */

#ifndef SCM_GSUBR_H
#define SCM_GSUBR_H

/* Copyright (C) 2002, 2007 Thien-Thi Nguyen
 * Copyright (C) 1995, 1996, 1998, 2000, 2002 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */




#define SCM_GSUBR_MAKTYPE(req, opt, rst) ((req)|((opt)<<4)|((rst)<<8))
#define SCM_GSUBR_REQ(x) ((int)(x)&0xf)
#define SCM_GSUBR_OPT(x) (((int)(x)&0xf0)>>4)
#define SCM_GSUBR_REST(x) ((int)(x)>>8)

#define SCM_GSUBR_MAX 10
#define SCM_GSUBR_TYPE(cclo) (SCM_VELTS (cclo)[1])
#define SCM_GSUBR_PROC(cclo) (SCM_VELTS (cclo)[2])

extern SCM scm_f_gsubr_apply;

extern SCM scm_make_gsubr (const char *name, int req, int opt, int rst,
                           SCM (*fcn)());
extern SCM scm_make_gsubr_with_generic (const char *name,
                                        int req,
                                        int opt,
                                        int rst,
                                        SCM (*fcn)(),
                                        SCM *gf);
extern SCM scm_gsubr_apply (SCM args);



/* Some (undocumented, internal to Guile) machinery to support exports
   extraction and the justly-vaunted "table-driven processing" paradigm.  */

typedef struct {
  const char *name;
  SCM (*fn)();
} scm_c2x_nm_fn_t;

#define SCM_C2X_MAKEALL_GSUBR(nmfn_table,rov_table)     \
  do {                                                  \
    scm_c2x_nm_fn_t *__nmfn = nmfn_table;               \
    unsigned short int *__rov = rov_table;              \
    for (; __nmfn->name; __nmfn++, __rov++)             \
      scm_make_gsubr (__nmfn->name,                     \
                      SCM_GSUBR_REQ (*__rov),           \
                      SCM_GSUBR_OPT (*__rov),           \
                      SCM_GSUBR_REST (*__rov),          \
                      __nmfn->fn);                      \
  } while (0)

#endif  /* SCM_GSUBR_H */

/* gsubr.h ends here */
