/* hooks.h */

#ifndef SCM_HOOKS_H
#define SCM_HOOKS_H

/* Copyright (C) 2002, 2007 Thien-Thi Nguyen
 * Copyright (C) 1995, 1996, 1999, 2000, 2002 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */



/*
 * C level hooks
 */

/*
 * The interface is designed for and- and or-type hooks which
 * both may want to indicate success/failure and return a result.
 */

typedef enum scm_c_hook_type_t {
  SCM_C_HOOK_NORMAL,
  SCM_C_HOOK_OR,
  SCM_C_HOOK_AND
} scm_c_hook_type_t;

typedef void  *(*scm_c_hook_function_t) (void *hook_data,
					 void *func_data,
					 void *data);

typedef struct scm_c_hook_entry_t {
  struct scm_c_hook_entry_t *next;
  scm_c_hook_function_t func;
  void *data;
} scm_c_hook_entry_t;

typedef struct scm_c_hook_t {
  scm_c_hook_entry_t *first;
  scm_c_hook_type_t type;
  void *data;
} scm_c_hook_t;

extern void scm_c_hook_init (scm_c_hook_t *hook,
			     void *hook_data,
			     scm_c_hook_type_t type);
extern void scm_c_hook_add (scm_c_hook_t *hook,
			    scm_c_hook_function_t func,
			    void *func_data,
			    int appendp);
extern void scm_c_hook_remove (scm_c_hook_t *hook,
			       scm_c_hook_function_t func,
			       void *func_data);
extern void *scm_c_hook_run (scm_c_hook_t *hook, void *data);

/*
 * Scheme level hooks
 */

#define SCM_HOOKP(x) (!SCM_IMP (x) && (SCM_TYP16 (x) == scm_tc16_hook))
#define SCM_HOOK_ARITY(hook) (SCM_CELL_WORD_0 (hook) >> 16)
#define SCM_HOOK_PROCEDURES(hook) SCM_CELL_OBJECT_1 (hook)
#define SCM_SET_HOOK_PROCEDURES(hook, procs) SCM_SET_CELL_OBJECT_1 ((hook), (procs))

extern long scm_tc16_hook;

extern SCM scm_make_hook (SCM n_args);
extern SCM scm_create_hook (const char* name, int n_args);
extern SCM scm_hook_p (SCM x);
extern SCM scm_hook_empty_p (SCM hook);
extern SCM scm_add_hook_x (SCM hook, SCM thunk, SCM appendp);
extern SCM scm_remove_hook_x (SCM hook, SCM thunk);
extern SCM scm_reset_hook_x (SCM hook);
extern SCM scm_run_hook (SCM hook, SCM args);
extern void scm_c_run_hook (SCM hook, SCM args);
extern SCM scm_hook_to_list (SCM hook);




#if (SCM_DEBUG_DEPRECATED == 0)

/* Use scm_set_object_property_x to set the name property of a hook: */
#define SCM_HOOK_NAME(h) scm_object_property (h, scm_makfrom0str ("name"))
extern SCM scm_make_hook_with_name (SCM name, SCM n_args);

#endif  /* SCM_DEBUG_DEPRECATED == 0 */

#endif  /* SCM_HOOKS_H */

/* hooks.h ends here */
