/* iselect.h */

#ifndef SCM_ISELECT_H
#define SCM_ISELECT_H

/* Copyright (C) 2002, 2007 Thien-Thi Nguyen
 * Copyright (C) 1997, 1998, 2000, 2002 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */


/* Needed for FD_SET on some systems.  */
#include <sys/types.h>

#if 1 /* petrified TIME_WITH_SYS_TIME */
#  include <sys/time.h>
#  include <time.h>
# else
#if 1 /* petrified HAVE_SYS_TIME_H */
#   include <sys/time.h>
#  else
#if 1 /* petrified HAVE_TIME_H */
#    include <time.h>
#   endif
#  endif
# endif

#if 1 /* petrified HAVE_SYS_SELECT_H */
#include <sys/select.h>
#endif

#ifdef FD_SET

#define SCM_SELECT_TYPE fd_set
#define SCM_SELECT_SET_SIZE FD_SETSIZE
#define SCM_FD_SET(n, p)   FD_SET (n, p)
#define SCM_FD_CLR(n, p)   FD_CLR (n, p)
#define SCM_FD_ISSET(n, p) FD_ISSET (n, p)
#define SCM_FD_ZERO(p)     FD_ZERO (p)

#else /* no FD_SET */

/* Define the macros to access a single-int bitmap of descriptors.  */
#define SCM_SELECT_TYPE int
#define SCM_SELECT_SET_SIZE 32
#define SCM_FD_SET(n, p) (*(p) |= (1 << (n)))
#define SCM_FD_CLR(n, p) (*(p) &= ~(1 << (n)))
#define SCM_FD_ISSET(n, p) (*(p) & (1 << (n)))
#define SCM_FD_ZERO(p) (*(p) = 0)

#endif /* no FD_SET */

extern int scm_internal_select (int fds,
				SCM_SELECT_TYPE *rfds,
				SCM_SELECT_TYPE *wfds,
				SCM_SELECT_TYPE *efds,
				struct timeval *timeout);

/* Provide SCM_SELECT abstraction.  */
#if 0 /* petrified GUILE_ISELECT */
# define SCM_SELECT scm_internal_select
#else
# define SCM_SELECT select
#endif

#endif /* SCM_ISELECT_H */

/* iselect.h ends here */
