/* random.h */

#ifndef SCM_RANDOM_H
#define SCM_RANDOM_H

/* Copyright (C) 2002, 2007 Thien-Thi Nguyen
 * Copyright (C) 1999, 2000, 2002 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */




/*
 * A plugin interface for RNGs
 *
 * Using this interface, it is possible for the application to tell
 * libguile to use a different RNG.  This is desirable if it is
 * necessary to use the same RNG everywhere in the application in
 * order to prevent interference, if the application uses RNG
 * hardware, or if the application has special demands on the RNG.
 *
 * Look how the default generator is "plugged in" in scm_init_random().
 */

typedef struct scm_rstate {
  int reserved0;
  double reserved1;
  /* Custom fields follow here */
} scm_rstate;

typedef struct scm_rng {
  size_t rstate_size;				    /* size of random state */
  unsigned long (*random_bits) (scm_rstate *state); /* gives 32 random bits */
  void (*init_rstate) (scm_rstate *state, char *seed, int n);
  scm_rstate *(*copy_rstate) (scm_rstate *state);
} scm_rng;

extern scm_rng scm_the_rng;


/*
 * Default RNG
 */
typedef struct scm_i_rstate {
  scm_rstate rstate;
  unsigned long w;
  unsigned long c;
} scm_i_rstate;

extern unsigned long scm_i_uniform32 (scm_i_rstate *);
extern void scm_i_init_rstate (scm_i_rstate *, char *seed, int n);
extern scm_i_rstate *scm_i_copy_rstate (scm_i_rstate *);


/*
 * Random number library functions
 */
extern scm_rstate *scm_c_make_rstate (char *, int);
extern scm_rstate *scm_c_default_rstate (void);
#define scm_c_uniform32(RSTATE) scm_the_rng.random_bits (RSTATE)
extern double scm_c_uniform01 (scm_rstate *);
extern double scm_c_normal01 (scm_rstate *);
extern double scm_c_exp1 (scm_rstate *);
extern unsigned long scm_c_random (scm_rstate *, unsigned long m);
extern SCM scm_c_random_bignum (scm_rstate *, SCM m);


/*
 * Scheme level interface
 */
extern long scm_tc16_rstate;
#define SCM_RSTATE(obj) ((scm_rstate *) SCM_CELL_WORD_1 (obj))
#define SCM_RSTATEP(obj) (SCM_NIMP (obj) && (SCM_TYP16 (obj) == scm_tc16_rstate))

extern unsigned char scm_masktab[256];

extern SCM scm_var_random_state;
extern SCM scm_random (SCM n, SCM state);
extern SCM scm_copy_random_state (SCM state);
extern SCM scm_seed_to_random_state (SCM seed);
extern SCM scm_random_uniform (SCM state);
extern SCM scm_random_solid_sphere_x (SCM v, SCM state);
extern SCM scm_random_hollow_sphere_x (SCM v, SCM state);
extern SCM scm_random_normal (SCM state);
extern SCM scm_random_normal_vector_x (SCM v, SCM state);
extern SCM scm_random_exp (SCM state);


#endif  /* SCM_RANDOM_H */

/* random.h ends here */
