/* throw.h */

#ifndef SCM_THROW_H
#define SCM_THROW_H

/* Copyright (C) 2002, 2007 Thien-Thi Nguyen
 * Copyright (C) 1995, 1996, 1998, 2000, 2002 Free Software Foundation, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */




typedef SCM (*scm_catch_body_t) (void *data);
typedef SCM (*scm_catch_handler_t) (void *data,
                                    SCM tag, SCM throw_args);

extern SCM scm_internal_catch (SCM tag,
                               scm_catch_body_t body,
                               void *body_data,
                               scm_catch_handler_t handler,
                               void *handler_data);

extern SCM scm_internal_lazy_catch (SCM tag,
                                    scm_catch_body_t body,
                                    void *body_data,
                                    scm_catch_handler_t handler,
                                    void *handler_data);

extern SCM scm_internal_stack_catch (SCM tag,
                                     scm_catch_body_t body,
                                     void *body_data,
                                     scm_catch_handler_t handler,
                                     void *handler_data);

/* The first argument to scm_body_thunk should be a pointer to one of
   these.  See the implementation of catch in throw.c.  */
struct scm_body_thunk_data
{
  /* The tag being caught.  We only use it to figure out what
     arguments to pass to the body procedure; see scm_catch_thunk_body for
     details.  */
  SCM tag;

  /* The Scheme procedure object constituting the catch body.
     scm_body_by_proc invokes this.  */
  SCM body_proc;
};

extern SCM scm_body_thunk (void *);


extern SCM scm_handle_by_proc (void *, SCM, SCM);
extern SCM scm_handle_by_proc_catching_all (void *, SCM, SCM);
extern SCM scm_handle_by_message (void *, SCM, SCM);
extern SCM scm_handle_by_message_noexit (void *, SCM, SCM);
extern SCM scm_handle_by_throw (void *, SCM, SCM);
extern int scm_exit_status (SCM args);

extern SCM scm_catch (SCM tag, SCM thunk, SCM handler);
extern SCM scm_lazy_catch (SCM tag, SCM thunk, SCM handler);
extern SCM scm_ithrow (SCM key, SCM args, int noreturn);

extern SCM scm_throw (SCM key, SCM args);


#endif  /* SCM_THROW_H */

/* throw.h ends here */
